@@ -11,7 +11,6 @@
 #ifdef HAVE_CONFIG2_H
 #include "config2.h"
 #endif
-#endif
 
 /* no one will probably ever port svgalib on atheos or beos or port atheos
    interface to beos, but anyway: make sure they don't clash */
@@ -30,3 +29,13 @@
 #undef GRDRV_SVGALIB
 #endif
 #endif
+
+#if !(defined(__EMX__) || defined(_WIN32) || defined(__CYGWIN__))
+#ifdef GRDRV_PMSHELL
+#undef GRDRV_PMSHELL
+#endif
+#endif
+
+typedef int cfg_h_no_empty_unit;
+
+#endif
