@@ -16,16 +16,6 @@
 #include <math.h>
 #endif /* HAVE_MATH_H */
 
-#endif
-int dither_letters=1;
-
-double user_gamma=1.0; /* 1.0 for 64 lx. This is the number user directly changes in the menu */
-			  
-double display_red_gamma=2.2; /* Red gamma exponent of the display */
-double display_green_gamma=2.2; /* Green gamma exponent of the display */
-double display_blue_gamma=2.2; /* Blue gamma exponent of the display */
-#ifdef G
-
 /* #define this if you want to report missing letters to stderr.
  * Leave it commented up for normal operation and releases! */
 /* #define REPORT_UNKNOWN 1 */
@@ -1050,9 +1040,9 @@ void apply_gamma_exponent_and_undercolor_64_to_48(unsigned short *dest,
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
 		/* To prevent segfaults in case of crappy floating arithmetics
 		 */
-		if (ri>=65536) ri=65535;
-		if (gi>=65536) gi=65535;
-		if (bi>=65536) bi=65535;
+		if (ri>=65535) ri=65535;
+		if (gi>=65535) gi=65535;
+		if (bi>=65535) bi=65535;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		if (((alpha+1U)&255U)>=2U){
 			calpha=65535U-alpha;
@@ -1130,21 +1120,21 @@ void apply_gamma_exponent_48_to_48(unsigned short *dest,
 		a=pow(a,red_gamma);
 		*dest=(a*65535)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-		if (*dest>=0x10000) *dest=0xffff;
+		if (*dest>=0xffff) *dest=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		a=src[1];
 		a*=inv_65535;
 		a=pow(a,green_gamma);
 		dest[1]=(a*65535)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-		if (dest[1]>=0x10000) dest[1]=0xffff;
+		if (dest[1]>=0xffff) dest[1]=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		a=src[2];
 		a*=inv_65535;
 		a=pow(a,blue_gamma);
 		dest[2]=(a*65535)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-		if (dest[2]>=0x10000) dest[2]=0xffff;
+		if (dest[2]>=0xffff) dest[2]=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 	}
 }
@@ -1182,21 +1172,21 @@ void apply_gamma_exponent_24_to_48(unsigned short *dest, unsigned char *src, int
 		a=pow(a,red_gamma);
 		*dest=(a*65535)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-		if (*dest>=0x10000) *dest=0xffff;
+		if (*dest>=0xffff) *dest=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		a=src[1];
 		a*=inv_255;
 		a=pow(a,green_gamma);
 		dest[1]=(a*65535)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-		if (dest[1]>=0x10000) dest[1]=0xffff;
+		if (dest[1]>=0xffff) dest[1]=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		a=src[2];
 		a*=inv_255;
 		a=pow(a,blue_gamma);
 		dest[2]=(a*65535)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-		if (dest[2]>=0x10000) dest[2]=0xffff;
+		if (dest[2]>=0xffff) dest[2]=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 	}
 }
@@ -1222,19 +1212,19 @@ void make_gamma_table(struct cached_image *cimg)
 			*ptr_16=65535*pow(((double)a)*inv,rg)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
 		/* To test against crappy arithmetics */
-			if (*ptr_16>=0x10000) *ptr_16=0xffff;
+			if (*ptr_16>=0xffff) *ptr_16=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		}
 		for (a=0;a<256;a++,ptr_16++){
 			*ptr_16=65535*pow(((double)a)*inv,gg)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-			if (*ptr_16>=0x10000) *ptr_16=0xffff;
+			if (*ptr_16>=0xffff) *ptr_16=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		}
 		for (a=0;a<256;a++,ptr_16++){
 			*ptr_16=65535*pow(((double)a)*inv,bg)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-			if (*ptr_16>=0x10000) *ptr_16=0xffff;
+			if (*ptr_16>=0xffff) *ptr_16=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		}
 	}else{
@@ -1245,19 +1235,19 @@ void make_gamma_table(struct cached_image *cimg)
 		for (a=0;a<0x10000;a++,ptr_16++){
 			*ptr_16=65535*pow(((double)a)*inv,rg)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-			if (*ptr_16>=0x10000) *ptr_16=0xffff;
+			if (*ptr_16>=0xffff) *ptr_16=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		}
 		for (a=0;a<0x10000;a++,ptr_16++){
 			*ptr_16=65535*pow(((double)a)*inv,gg)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-			if (*ptr_16>=0x10000) *ptr_16=0xffff;
+			if (*ptr_16>=0xffff) *ptr_16=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		}
 		for (a=0;a<0x10000;a++,ptr_16++){
 			*ptr_16=65535*pow(((double)a)*inv,bg)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-			if (*ptr_16>=0x10000) *ptr_16=0xffff;
+			if (*ptr_16>=0xffff) *ptr_16=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 		}
 	}
@@ -1295,7 +1285,7 @@ unsigned short apply_gamma_single_8_to_16(unsigned char input, float gamma)
 	a*=65535;
 	retval = a+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-			if (retval>=0x10000) retval=0xffff;
+			if (retval>=0xffff) retval=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 	return retval;
 }
@@ -1314,13 +1304,12 @@ unsigned short apply_gamma_single_16_to_16(unsigned short input, float gamma)
 	
 	retval = 65535*pow(((float)input)/65535,gamma)+0.5;
 #if !SIZEOF_UNSIGNED_SHORT || SIZEOF_UNSIGNED_SHORT > 2
-			if (retval>=0x10000) retval=0xffff;
+			if (retval>=0xffff) retval=0xffff;
 #endif /* #if SIZEOF_UNSIGNED_SHORT > 2 */
 	return retval;
 }
 
 /* Points to the next unread byte from png data block */
-extern unsigned char font_data[];
 extern struct letter letter_data[];
 extern struct font font_table[];
 extern int n_fonts; /* Number of fonts. font number 0 is system_font (it's
@@ -1427,18 +1416,14 @@ unsigned char *png_data, int png_length, struct style *style)
 	png_ptr=png_create_read_struct(PNG_LIBPNG_VER_STRING,
 			NULL, my_png_error, my_png_warning);
 	if (!png_ptr) {
-		if (out_of_memory(NULL, 0)) goto retry1;
-		error("png_create_read_struct failed");
-		fatal_tty_exit();
-		exit(RET_FATAL);
+		if (out_of_memory(0, NULL, 0)) goto retry1;
+		fatal_exit("png_create_read_struct failed");
 	}
 	retry2:
 	info_ptr=png_create_info_struct(png_ptr);
 	if (!info_ptr) {
-		if (out_of_memory(NULL, 0)) goto retry2;
-		error("png_create_info_struct failed");
-		fatal_tty_exit();
-		exit(RET_FATAL);
+		if (out_of_memory(0, NULL, 0)) goto retry2;
+		fatal_exit("png_create_info_struct failed");
 	}
 	png_set_read_fn(png_ptr,&work,(png_rw_ptr)&read_stored_data);
 	png_read_info(png_ptr, info_ptr);
@@ -1597,7 +1582,7 @@ static struct font_cache_entry *supply_color_cache_entry (struct graphics_driver
 		found=mem_alloc(sizeof(*found));
 		found->bitmap.y=style->height;
 		load_scaled_char(&(found->bitmap.data),&(found->bitmap.x),
-			found->bitmap.y, font_data+letter->begin,
+			found->bitmap.y, letter->begin,
 			letter->length, style);
 		do_free=1;
 	} else {
@@ -1984,7 +1969,7 @@ void init_dip(void)
 {
 	init_font_cache();
 	update_aspect();
-	register_cache_upcall(shrink_font_cache, "fontcache");
+	register_cache_upcall(shrink_font_cache, MF_GPI, cast_uchar "fontcache");
 }
 
 static void recode_font_name(unsigned char **name)
@@ -1992,15 +1977,15 @@ static void recode_font_name(unsigned char **name)
 	int dashes=0;
 	unsigned char *p;
 
-	if (!strcmp(*name,"monospaced")) *name="courier-medium-roman-serif-mono";
-	if (!strcmp(*name,"monospace")) *name="courier-medium-roman-serif-mono";
-	else if (!strcmp(*name,"")) *name="century_school-medium-roman-serif-vari";
+	if (!strcmp(cast_const_char *name,"monospaced")) *name=cast_uchar "courier-medium-roman-serif-mono";
+	if (!strcmp(cast_const_char *name,"monospace")) *name=cast_uchar "courier-medium-roman-serif-mono";
+	else if (!strcmp(cast_const_char *name,"")) *name=cast_uchar "century_school-medium-roman-serif-vari";
 	p=*name;
 	while(*p){
 		if (*p=='-')dashes++;
 		p++;
 	}
-	if (dashes!=4) *name="century_school-medium-roman-serif-vari";
+	if (dashes!=4) *name=cast_uchar "century_school-medium-roman-serif-vari";
 }
 
 /* Compares single=a multi=b-c-a as matching.
@@ -2010,17 +1995,17 @@ static void recode_font_name(unsigned char **name)
 static int compare_family(unsigned char *single, unsigned char *multi)
 {
 	unsigned char *p,*r;
-	int single_length=strlen(single);
+	int single_length=strlen(cast_const_char single);
 
 	r=multi;
 	while(1){
 		p=r;
 		while (*r&&*r!='-')r++;
-		if ((r-p==single_length)&&!strncmp(single,p,r-p)) return 0;
+		if ((r-p==single_length)&&!strncmp(cast_const_char single,cast_const_char p,r-p)) return 0;
 		if (!*r) return 1;
 		r++;
 	}
-	return 1;
+	not_reached(return 1;)
 }
 
 /* Input name must contain exactly 4 dashes, otherwise the
@@ -2058,20 +2043,20 @@ static int fill_style_table(int * table, unsigned char *name)
 	*p=0;
 	p++;
 	spacing=p;
-	monospaced=!strcmp(spacing,"mono");
+	monospaced=!strcmp(cast_const_char spacing,"mono");
 	
 	for (pass=0;pass<6;pass++){
 		for (f=1;f<n_fonts;f++){
 			/* Font 0 must not be int style_table */
 			result=compare_family(family,font_table[f].family);
 			result<<=1;
-			result|=!!strcmp(weight,font_table[f].weight);
+			result|=!!strcmp(cast_const_char weight,cast_const_char font_table[f].weight);
 			result<<=1;
-			result|=!!strcmp(slant,font_table[f].slant);
+			result|=!!strcmp(cast_const_char slant,cast_const_char font_table[f].slant);
 			result<<=1;
-			result|=!!strcmp(adstyl,font_table[f].adstyl);
+			result|=!!strcmp(cast_const_char adstyl,cast_const_char font_table[f].adstyl);
 			result<<=1;
-			result|=!!strcmp(spacing,font_table[f].spacing);
+			result|=!!strcmp(cast_const_char spacing,cast_const_char font_table[f].spacing);
 			result^=xors[pass];
 			result&=masks[pass];
 			if (!result) /* Fot complies */
@@ -2181,7 +2166,7 @@ long real_dip_get_color_sRGB(int rgb)
 
 /* ATTENTION!!! allocates using malloc. Due to braindead Xlib, which
  * frees it using free and thus it is not possible to use mem_alloc. */
-void get_links_icon(unsigned char **data, int *width, int *height, int depth)
+void get_links_icon(unsigned char **data, int *width, int *height, int *skip, int pad)
 {
 	struct bitmap b;
 	unsigned short *tmp1;
@@ -2191,10 +2176,12 @@ void get_links_icon(unsigned char **data, int *width, int *height, int depth)
 	b.y=48;
 	*width=b.x;
 	*height=b.y;
-	b.skip=b.x*(depth&7);
+	b.skip=b.x*(drv->depth&7);
+	while (b.skip % pad) b.skip++;
+	*skip=b.skip;
 	retry:
 	if (!(b.data=*data=malloc(b.skip*b.y))) {
-		out_of_memory("icon malloc", b.skip*b.y);
+		out_of_memory(0, cast_uchar "icon malloc", b.skip*b.y);
 		goto retry;
 	}
 	tmp1=mem_alloc(6*b.y*b.x);
