@@ -44,12 +44,12 @@
 static void get_align(unsigned char *attr, int *a)
 {
 	unsigned char *al;
-	if ((al = get_attr_val(attr, "align"))) {
-		if (!(strcasecmp(al, "left"))) *a = AL_LEFT;
-		if (!(strcasecmp(al, "right"))) *a = AL_RIGHT;
-		if (!(strcasecmp(al, "center"))) *a = AL_CENTER;
-		if (!(strcasecmp(al, "justify"))) *a = AL_BLOCK;
-		if (!(strcasecmp(al, "char"))) *a = AL_RIGHT; /* NOT IMPLEMENTED */
+	if ((al = get_attr_val(attr, cast_uchar "align"))) {
+		if (!(strcasecmp(cast_const_char al, "left"))) *a = AL_LEFT;
+		if (!(strcasecmp(cast_const_char al, "right"))) *a = AL_RIGHT;
+		if (!(strcasecmp(cast_const_char al, "center"))) *a = AL_CENTER;
+		if (!(strcasecmp(cast_const_char al, "justify"))) *a = AL_BLOCK;
+		if (!(strcasecmp(cast_const_char al, "char"))) *a = AL_RIGHT; /* NOT IMPLEMENTED */
 		mem_free(al);
 	}
 }
@@ -57,11 +57,11 @@ static void get_align(unsigned char *attr, int *a)
 static void get_valign(unsigned char *attr, int *a)
 {
 	unsigned char *al;
-	if ((al = get_attr_val(attr, "valign"))) {
-		if (!(strcasecmp(al, "top"))) *a = VAL_TOP;
-		if (!(strcasecmp(al, "middle"))) *a = VAL_MIDDLE;
-		if (!(strcasecmp(al, "bottom"))) *a = VAL_BOTTOM;
-		if (!(strcasecmp(al, "baseline"))) *a = VAL_TOP; /* NOT IMPLEMENTED */
+	if ((al = get_attr_val(attr, cast_uchar "valign"))) {
+		if (!(strcasecmp(cast_const_char al, "top"))) *a = VAL_TOP;
+		if (!(strcasecmp(cast_const_char al, "middle"))) *a = VAL_MIDDLE;
+		if (!(strcasecmp(cast_const_char al, "bottom"))) *a = VAL_BOTTOM;
+		if (!(strcasecmp(cast_const_char al, "baseline"))) *a = VAL_TOP; /* NOT IMPLEMENTED */
 		mem_free(al);
 	}
 }
@@ -69,15 +69,15 @@ static void get_valign(unsigned char *attr, int *a)
 static void get_c_width(unsigned char *attr, int *w, int sh)
 {
 	unsigned char *al;
-	if ((al = get_attr_val(attr, "width"))) {
-		if (*al && al[strlen(al) - 1] == '*') {
+	if ((al = get_attr_val(attr, cast_uchar "width"))) {
+		if (*al && al[strlen(cast_const_char al) - 1] == '*') {
 			unsigned char *en;
 			unsigned long n;
-			al[strlen(al) - 1] = 0;
-			n = strtoul(al, (char **)(void *)&en, 10);
+			al[strlen(cast_const_char al) - 1] = 0;
+			n = strtoul(cast_const_char al, (char **)(void *)&en, 10);
 			if (n < 10000 && !*en) *w = W_REL - n;
 		} else {
-			int p = get_width(attr, "width", sh);
+			int p = get_width(attr, cast_uchar "width", sh);
 			if (p >= 0) *w = p;
 		}
 		mem_free(al);
@@ -366,7 +366,7 @@ static void set_td_width(struct table *t, int x, int width, int f)
 
 unsigned char *skip_element(unsigned char *html, unsigned char *eof, unsigned char *what, int sub)
 {
-	int l = strlen(what);
+	int l = strlen(cast_const_char what);
 	int level = 1;
 	unsigned char *name;
 	int namelen;
@@ -442,21 +442,21 @@ static struct table *parse_table(unsigned char *html, unsigned char *eof, unsign
 		html++;
 		goto se;
 	}
-	if (t_namelen == 5 && !casecmp(t_name, "TABLE", 5)) {
-		en = skip_element(en, eof, "TABLE", 1);
+	if (t_namelen == 5 && !casecmp(t_name, cast_uchar "TABLE", 5)) {
+		en = skip_element(en, eof, cast_uchar "TABLE", 1);
 		goto see;
 	}
-	if (t_namelen == 6 && !casecmp(t_name, "SCRIPT", 5)) {
-		en = skip_element(en, eof, "SCRIPT", 0);
+	if (t_namelen == 6 && !casecmp(t_name, cast_uchar "SCRIPT", 5)) {
+		en = skip_element(en, eof, cast_uchar "SCRIPT", 0);
 		goto see;
 	}
-	if (t_namelen == 6 && !casecmp(t_name, "/TABLE", 6)) {
+	if (t_namelen == 6 && !casecmp(t_name, cast_uchar "/TABLE", 6)) {
 		if (c_span) new_columns(t, c_span, c_width, c_al, c_val, 1);
 		if (p) CELL(t, x, y)->end = html;
 		if (lbhp) (*bad_html)[*bhp-1].e = html;
 		goto scan_done;
 	}
-	if (t_namelen == 8 && !casecmp(t_name, "COLGROUP", 8)) {
+	if (t_namelen == 8 && !casecmp(t_name, cast_uchar "COLGROUP", 8)) {
 		if (c_span) new_columns(t, c_span, c_width, c_al, c_val, 1);
 		if (lbhp) (*bad_html)[*bhp-1].e = html, lbhp = NULL;
 		c_al = AL_TR;
@@ -465,10 +465,10 @@ static struct table *parse_table(unsigned char *html, unsigned char *eof, unsign
 		get_align(t_attr, &c_al);
 		get_valign(t_attr, &c_val);
 		get_c_width(t_attr, &c_width, sh);
-		if ((c_span = get_num(t_attr, "span")) == -1) c_span = 1;
+		if ((c_span = get_num(t_attr, cast_uchar "span")) == -1) c_span = 1;
 		goto see;
 	}
-	if (t_namelen == 9 && !casecmp(t_name, "/COLGROUP", 9)) {
+	if (t_namelen == 9 && !casecmp(t_name, cast_uchar "/COLGROUP", 9)) {
 		if (c_span) new_columns(t, c_span, c_width, c_al, c_val, 1);
 		if (lbhp) (*bad_html)[*bhp-1].e = html, lbhp = NULL;
 		c_span = 0;
@@ -477,10 +477,10 @@ static struct table *parse_table(unsigned char *html, unsigned char *eof, unsign
 		c_width = W_AUTO;
 		goto see;
 	}
-	if (t_namelen == 3 && !casecmp(t_name, "COL", 3)) {
+	if (t_namelen == 3 && !casecmp(t_name, cast_uchar "COL", 3)) {
 		int sp, wi, al, val;
 		if (lbhp) (*bad_html)[*bhp-1].e = html, lbhp = NULL;
-		if ((sp = get_num(t_attr, "span")) == -1) sp = 1;
+		if ((sp = get_num(t_attr, cast_uchar "span")) == -1) sp = 1;
 		wi = c_width;
 		al = c_al;
 		val = c_val;
@@ -491,12 +491,12 @@ static struct table *parse_table(unsigned char *html, unsigned char *eof, unsign
 		c_span = 0;
 		goto see;
 	}
-	if (t_namelen == 3 && (!casecmp(t_name, "/TR", 3) || !casecmp(t_name, "/TD", 3) || !casecmp(t_name, "/TH", 3))) {
+	if (t_namelen == 3 && (!casecmp(t_name, cast_uchar "/TR", 3) || !casecmp(t_name, cast_uchar "/TD", 3) || !casecmp(t_name, cast_uchar "/TH", 3))) {
 		if (c_span) new_columns(t, c_span, c_width, c_al, c_val, 1);
 		if (p) CELL(t, x, y)->end = html, p = 0;
 		if (lbhp) (*bad_html)[*bhp-1].e = html, lbhp = NULL;
 	}
-	if (t_namelen == 2 && !casecmp(t_name, "TR", 2)) {
+	if (t_namelen == 2 && !casecmp(t_name, cast_uchar "TR", 2)) {
 		if (c_span) new_columns(t, c_span, c_width, c_al, c_val, 1);
 		if (p) CELL(t, x, y)->end = html, p = 0;
 		if (lbhp) (*bad_html)[*bhp-1].e = html, lbhp = NULL;
@@ -510,12 +510,12 @@ static struct table *parse_table(unsigned char *html, unsigned char *eof, unsign
 		y++, x = 0;
 		goto see;
 	}
-	if (t_namelen == 5 && ((!casecmp(t_name, "THEAD", 5)) || (!casecmp(t_name, "TBODY", 5)) || (!casecmp(t_name, "TFOOT", 5)))) {
+	if (t_namelen == 5 && ((!casecmp(t_name, cast_uchar "THEAD", 5)) || (!casecmp(t_name, cast_uchar "TBODY", 5)) || (!casecmp(t_name, cast_uchar "TFOOT", 5)))) {
 		if (c_span) new_columns(t, c_span, c_width, c_al, c_val, 1);
 		if (lbhp) (*bad_html)[*bhp-1].e = html, lbhp = NULL;
 		group = 2;
 	}
-	if (t_namelen != 2 || (casecmp(t_name, "TD", 2) && casecmp(t_name, "TH", 2))) goto see;
+	if (t_namelen != 2 || (casecmp(t_name, cast_uchar "TD", 2) && casecmp(t_name, cast_uchar "TH", 2))) goto see;
 	if (c_span) new_columns(t, c_span, c_width, c_al, c_val, 1);
 	if (lbhp) (*bad_html)[*bhp-1].e = html, lbhp = NULL;
 	if (p) CELL(t, x, y)->end = html, p = 0;
@@ -539,7 +539,7 @@ static struct table *parse_table(unsigned char *html, unsigned char *eof, unsign
 	if (upcase(t_name[1]) == 'H') {
 		unsigned char *e = en;
 		while (e < eof && WHITECHAR(*e)) e++;
-		if (eof - e > 6 && !casecmp(e, "<TABLE", 6)) goto no_th; /* hack for www.root.cz */
+		if (eof - e > 6 && !casecmp(e, cast_uchar "<TABLE", 6)) goto no_th; /* hack for www.root.cz */
 		cell->b = 1;
 		cell->align = AL_CENTER;
 		no_th:;
@@ -555,11 +555,11 @@ static struct table *parse_table(unsigned char *html, unsigned char *eof, unsign
 	get_valign(t_attr, &cell->valign);
 	get_bgcolor(t_attr, &cell->bgcolor);
 #ifdef G
-	sprintf(cell->bgcolor_str, "#%02x%02x%02x", cell->bgcolor.r & 0xff, cell->bgcolor.g & 0xff, cell->bgcolor.b & 0xff);
+	sprintf(cast_char cell->bgcolor_str, "#%02x%02x%02x", cell->bgcolor.r & 0xff, cell->bgcolor.g & 0xff, cell->bgcolor.b & 0xff);
 #endif
-	if ((csp = get_num(t_attr, "colspan")) == -1) csp = 1;
+	if ((csp = get_num(t_attr, cast_uchar "colspan")) == -1) csp = 1;
 	if (!csp) csp = -1;
-	if ((rsp = get_num(t_attr, "rowspan")) == -1) rsp = 1;
+	if ((rsp = get_num(t_attr, cast_uchar "rowspan")) == -1) rsp = 1;
 	if (!rsp) rsp = -1;
 	if (csp >= 0 && rsp >= 0 && csp * rsp > 100000) {
 		if (csp > 10) csp = -1;
@@ -1347,10 +1347,10 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 		/*fg = fg_color(fg, bg);*/
 		AF = ATTR_FRAME | (fg & 7) | (bg << 3) | ((fg & 8) << 3);
 	}
-	if ((border = get_num(attr, "border")) == -1) border = has_attr(attr, "border") || has_attr(attr, "rules") || has_attr(attr, "frame");
+	if ((border = get_num(attr, cast_uchar "border")) == -1) border = has_attr(attr, cast_uchar "border") || has_attr(attr, cast_uchar "rules") || has_attr(attr, cast_uchar "frame");
 	/*if (!border) border = 1;*/
-	if ((cellsp = get_num(attr, "cellspacing")) == -1) cellsp = gf_val(1, 2);
-	if ((cellpd = get_num(attr, "cellpadding")) == -1) {
+	if ((cellsp = get_num(attr, cast_uchar "cellspacing")) == -1) cellsp = gf_val(1, 2);
+	if ((cellpd = get_num(attr, cast_uchar "cellpadding")) == -1) {
 		vcellpd = gf_val(0, 1);
 		cellpd = gf_val(!!border, 1);
 	} else {
@@ -1370,37 +1370,37 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 #endif
 	align = par_format.align;
 	if (align == AL_NO || align == AL_BLOCK) align = AL_LEFT;
-	if ((al = get_attr_val(attr, "align"))) {
-		if (!strcasecmp(al, "left")) align = AL_LEFT;
-		if (!strcasecmp(al, "center")) align = AL_CENTER;
-		if (!strcasecmp(al, "right")) align = AL_RIGHT;
+	if ((al = get_attr_val(attr, cast_uchar "align"))) {
+		if (!strcasecmp(cast_const_char al, "left")) align = AL_LEFT;
+		if (!strcasecmp(cast_const_char al, "center")) align = AL_CENTER;
+		if (!strcasecmp(cast_const_char al, "right")) align = AL_RIGHT;
 		mem_free(al);
 	}
 	frame = F_BOX;
-	if ((al = get_attr_val(attr, "frame"))) {
-		if (!strcasecmp(al, "void")) frame = F_VOID;
-		if (!strcasecmp(al, "above")) frame = F_ABOVE;
-		if (!strcasecmp(al, "below")) frame = F_BELOW;
-		if (!strcasecmp(al, "hsides")) frame = F_HSIDES;
-		if (!strcasecmp(al, "vsides")) frame = F_VSIDES;
-		if (!strcasecmp(al, "lhs")) frame = F_LHS;
-		if (!strcasecmp(al, "rhs")) frame = F_RHS;
-		if (!strcasecmp(al, "box")) frame = F_BOX;
-		if (!strcasecmp(al, "border")) frame = F_BOX;
+	if ((al = get_attr_val(attr, cast_uchar "frame"))) {
+		if (!strcasecmp(cast_const_char al, "void")) frame = F_VOID;
+		if (!strcasecmp(cast_const_char al, "above")) frame = F_ABOVE;
+		if (!strcasecmp(cast_const_char al, "below")) frame = F_BELOW;
+		if (!strcasecmp(cast_const_char al, "hsides")) frame = F_HSIDES;
+		if (!strcasecmp(cast_const_char al, "vsides")) frame = F_VSIDES;
+		if (!strcasecmp(cast_const_char al, "lhs")) frame = F_LHS;
+		if (!strcasecmp(cast_const_char al, "rhs")) frame = F_RHS;
+		if (!strcasecmp(cast_const_char al, "box")) frame = F_BOX;
+		if (!strcasecmp(cast_const_char al, "border")) frame = F_BOX;
 		mem_free(al);
 	}
 	rules = border ? R_ALL : R_NONE;
-	if ((al = get_attr_val(attr, "rules"))) {
-		if (!strcasecmp(al, "none")) rules = R_NONE;
-		if (!strcasecmp(al, "groups")) rules = R_GROUPS;
-		if (!strcasecmp(al, "rows")) rules = R_ROWS;
-		if (!strcasecmp(al, "cols")) rules = R_COLS;
-		if (!strcasecmp(al, "all")) rules = R_ALL;
+	if ((al = get_attr_val(attr, cast_uchar "rules"))) {
+		if (!strcasecmp(cast_const_char al, "none")) rules = R_NONE;
+		if (!strcasecmp(cast_const_char al, "groups")) rules = R_GROUPS;
+		if (!strcasecmp(cast_const_char al, "rows")) rules = R_ROWS;
+		if (!strcasecmp(cast_const_char al, "cols")) rules = R_COLS;
+		if (!strcasecmp(cast_const_char al, "all")) rules = R_ALL;
 		mem_free(al);
 	}
 	if (!border) frame = F_VOID;
 	wf = 0;
-	if ((width = get_width(attr, "width", gf_val(p->data || p->xp, !!gp->data))) == -1) {
+	if ((width = get_width(attr, cast_uchar "width", gf_val(p->data || p->xp, !!gp->data))) == -1) {
 		width = par_format.width - (par_format.leftmargin + par_format.rightmargin) * gf_val(1, G_HTML_MARGIN);
 		if (width < 0) width = 0;
 		wf = 1;
@@ -1426,7 +1426,7 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	{
 		t->p = p;
 	}
-	t->bordercolor = get_attr_val(attr, "bordercolor");
+	t->bordercolor = get_attr_val(attr, cast_uchar "bordercolor");
 	t->align = align;
 	t->border = border;
 	t->cellpd = cellpd;
@@ -1656,14 +1656,14 @@ static void table_get_list(struct g_object_table *o, void (*fn)(struct g_object
 
 void table_bg(struct text_attrib *ta, unsigned char bgstr[8])
 {
-	if (ta->bg.r + ta->bg.g * 3 + ta->bg.b * 5 > 9 * 128) strcpy(bgstr, "#000000");
+	if (ta->bg.r + ta->bg.g * 3 + ta->bg.b * 5 >= 9 * 128) strcpy(cast_char bgstr, "#000000");
 	else if (ta->fg.r > G_HTML_TABLE_FRAME_COLOR && ta->fg.g > G_HTML_TABLE_FRAME_COLOR && ta->fg.b > G_HTML_TABLE_FRAME_COLOR) {
 		unsigned char max = ta->fg.r;
 		if (ta->fg.g > max) max = ta->fg.g;
 		if (ta->fg.b > max) max = ta->fg.b;
 		max &= 0xff;
-		sprintf(bgstr, "#%02x%02x%02x", max, max, max);
-	} else sprintf(bgstr, "#%02x%02x%02x", G_HTML_TABLE_FRAME_COLOR, G_HTML_TABLE_FRAME_COLOR, G_HTML_TABLE_FRAME_COLOR);
+		sprintf(cast_char bgstr, "#%02x%02x%02x", max, max, max);
+	} else sprintf(cast_char bgstr, "#%02x%02x%02x", G_HTML_TABLE_FRAME_COLOR, G_HTML_TABLE_FRAME_COLOR, G_HTML_TABLE_FRAME_COLOR);
 }
 
 static void process_g_table(struct g_part *gp, struct table *t)
