@@ -10,6 +10,8 @@ static tcount connection_count = 0;
 
 static int active_connections = 0;
 
+tcount netcfg_stamp = 0;
+
 struct list_head queue = {&queue, &queue};
 
 struct h_conn {
@@ -66,7 +68,7 @@ static struct h_conn *is_host_on_list(struct connection *c)
 	unsigned char *ho;
 	struct h_conn *h;
 	if (!(ho = get_host_name(c->url))) return NULL;
-	foreach(h, h_conns) if (!strcmp(h->host, ho)) {
+	foreach(h, h_conns) if (!strcmp(cast_const_char h->host, cast_const_char ho)) {
 		mem_free(ho);
 		return h;
 	}
@@ -88,7 +90,7 @@ static void stat_timer(struct connection *c)
 		r->dis_b += (uttime)a;
 		while (r->dis_b >= SPD_DISP_TIME * CURRENT_SPD_SEC) {
 			r->cur_loaded -= r->data_in_secs[0];
-			memmove(r->data_in_secs, r->data_in_secs + 1, sizeof(int) * (CURRENT_SPD_SEC - 1));
+			memmove(r->data_in_secs, r->data_in_secs + 1, sizeof(off_t) * (CURRENT_SPD_SEC - 1));
 			r->data_in_secs[CURRENT_SPD_SEC - 1] = 0;
 			r->dis_b -= (uttime)SPD_DISP_TIME;
 		}
@@ -142,7 +144,7 @@ static struct k_conn *is_host_on_keepalive_list(struct connection *c)
 	if (!(ph = get_protocol_handle(c->url))) return NULL;
 	if (!(ho = get_host_and_pass(c->url))) return NULL;
 	foreach(h, keepalive_connections)
-		if (h->protocol == ph && h->port == po && !strcmp(h->host, ho)) {
+		if (h->protocol == ph && h->port == po && !strcmp(cast_const_char h->host, cast_const_char ho)) {
 			mem_free(ho);
 			return h;
 		}
@@ -269,10 +271,13 @@ void add_keepalive_socket(struct connection *c, ttime timeout)
 		goto del;
 	}
 	k = mem_alloc(sizeof(struct k_conn));
-	if ((k->port = get_port(c->url)) == -1 || !(k->protocol = get_protocol_handle(c->url)) || !(k->host = get_host_and_pass(c->url))) {
+	if (c->netcfg_stamp != netcfg_stamp ||
+	    (k->port = get_port(c->url)) == -1 ||
+	    !(k->protocol = get_protocol_handle(c->url)) ||
+	    !(k->host = get_host_and_pass(c->url))) {
 		mem_free(k);
 		del_connection(c);
-		goto close;
+		goto clos;
 	}
 	k->conn = c->sock1;
 	k->timeout = timeout;
@@ -285,7 +290,7 @@ void add_keepalive_socket(struct connection *c, ttime timeout)
 #endif
 	register_bottom_half(check_queue, NULL);
 	return;
-	close:
+	clos:
 	EINTRLOOP(rs, close(c->sock1));
 #ifdef DEBUG
 	check_queue_bugs();
@@ -383,7 +388,7 @@ static int try_to_suspend_connection(struct connection *c, unsigned char *ho)
 		if (ho) {
 			unsigned char *h;
 			if (!(h = get_host_name(d->url))) continue;
-			if (strcmp(h, ho)) {
+			if (strcmp(cast_const_char h, cast_const_char ho)) {
 				mem_free(h);
 				continue;
 			}
@@ -404,32 +409,26 @@ static void run_connection(struct connection *c)
 		return;
 	}
 
-	if (is_url_blocked(c->url)) {
-		setcstate(c, S_BLOCKED_URL);
-		del_connection(c);
-		return;
-	}
-
 	safe_strncpy(c->socks_proxy, proxies.socks_proxy, sizeof c->socks_proxy);
 
-	if (proxies.only_proxies && casecmp(c->url, "proxy://", 8) && (!*c->socks_proxy || url_bypasses_socks(c->url))) {
+	if (proxies.only_proxies && casecmp(c->url, cast_uchar "proxy://", 8) && (!*c->socks_proxy || url_bypasses_socks(c->url))) {
 		setcstate(c, S_NO_PROXY);
 		del_connection(c);
 		return;
 	}
 	
 	if (!(func = get_protocol_handle(c->url))) {
-		setcstate(c, S_BAD_URL);
+		s_bad_url:
+		if (!casecmp(c->url, cast_uchar "proxy://", 8)) setcstate(c, S_BAD_PROXY);
+		else setcstate(c, S_BAD_URL);
 		del_connection(c);
 		return;
 	}
 	if (!(hc = is_host_on_list(c))) {
 		hc = mem_alloc(sizeof(struct h_conn));
 		if (!(hc->host = get_host_name(c->url))) {
-			setcstate(c, S_BAD_URL);
-			del_connection(c);
 			mem_free(hc);
-			return;
+			goto s_bad_url;
 		}
 		hc->conn = 0;
 		add_to_list(h_conns, hc);
@@ -443,20 +442,21 @@ static void run_connection(struct connection *c)
 static int is_connection_seekable(struct connection *c)
 {
 	unsigned char *protocol = get_protocol_name(c->url);
-	if (!strcasecmp(protocol, "http") || !strcasecmp(protocol, "https") ||
-	    !strcasecmp(protocol, "proxy")) {
+	if (!strcasecmp(cast_const_char protocol, "http") ||
+	    !strcasecmp(cast_const_char protocol, "https") ||
+	    !strcasecmp(cast_const_char protocol, "proxy")) {
 		unsigned char *d;
 		mem_free(protocol);
 		if (!c->cache || !c->cache->head)
 			return 1;
-		d = parse_http_header(c->cache->head, "Accept-Ranges", NULL);
+		d = parse_http_header(c->cache->head, cast_uchar "Accept-Ranges", NULL);
 		if (d) {
 			mem_free(d);
 			return 1;
 		}
 		return 0;
 	}
-	if (!strcasecmp(protocol, "ftp")) {
+	if (!strcasecmp(cast_const_char protocol, "ftp")) {
 		mem_free(protocol);
 		return 1;
 	}
@@ -600,17 +600,25 @@ void check_queue(void *dummy)
 #endif
 }
 
+unsigned char *get_proxy_string(unsigned char *url)
+{
+	if (*proxies.http_proxy && !casecmp(url, cast_uchar "http://", 7)) return proxies.http_proxy;
+	if (*proxies.ftp_proxy && !casecmp(url, cast_uchar "ftp://", 6)) return proxies.ftp_proxy;
+#ifdef HAVE_SSL
+	if (*proxies.https_proxy && !casecmp(url, cast_uchar "https://", 8)) return proxies.https_proxy;
+#endif
+	return NULL;
+}
+
 unsigned char *get_proxy(unsigned char *url)
 {
-	size_t l = strlen(url);
-	unsigned char *proxy = NULL;
+	size_t l = strlen(cast_const_char url);
+	unsigned char *proxy = get_proxy_string(url);
 	unsigned char *u;
-	if (*proxies.http_proxy && l >= 7 && !casecmp(url, "http://", 7)) proxy = proxies.http_proxy;
-	if (*proxies.ftp_proxy && l >= 6 && !casecmp(url, "ftp://", 6)) proxy = proxies.ftp_proxy;
-	u = mem_alloc(l + 1 + (proxy ? strlen(proxy) + 9 : 0));
-	if (proxy) strcpy(u, "proxy://"), strcat(u, proxy), strcat(u, "/");
+	u = mem_alloc(l + 1 + (proxy ? strlen(cast_const_char proxy) + 9 : 0));
+	if (proxy) strcpy(cast_char u, "proxy://"), strcat(cast_char u, cast_const_char proxy), strcat(cast_char u, "/");
 	else *u = 0;
-	strcat(u, url);
+	strcat(cast_char u, cast_const_char url);
 	return u;
 }
 
@@ -622,7 +630,13 @@ void load_url(unsigned char *url, unsigned char *prev_url, struct status *stat,
 	struct connection *c;
 	unsigned char *u;
 	int must_detach = 0;
-	if (stat) stat->c = NULL, stat->ce = NULL, stat->pri = pri;
+	if (stat) {
+		stat->c = NULL;
+		stat->ce = NULL;
+		stat->state = S_OUT_OF_MEM;
+		stat->prev_error = 0;
+		stat->pri = pri;
+	}
 #ifdef DEBUG
 	foreach(c, queue) {
 		struct status *st;
@@ -636,7 +650,13 @@ void load_url(unsigned char *url, unsigned char *prev_url, struct status *stat,
 		}
 	}
 #endif
-	if (stat) stat->state = S_OUT_OF_MEM, stat->prev_error = 0;
+	if (is_url_blocked(url)) {
+		if (stat) {
+			stat->state = S_BLOCKED_URL;
+			if (stat->end) stat->end(stat, stat->data);
+		}
+		return;
+	}
 	if (no_cache <= NC_CACHE && !find_in_cache(url, &e)) {
 		if (e->incomplete) {
 			e->refcount--;
@@ -651,7 +671,7 @@ void load_url(unsigned char *url, unsigned char *prev_url, struct status *stat,
 		}
 		if (no_compress) {
 			unsigned char *enc;
-			enc = parse_http_header(e->head, "Content-Encoding", NULL);
+			enc = parse_http_header(e->head, cast_uchar "Content-Encoding", NULL);
 			if (enc) {
 				mem_free(enc);
 				e->refcount--;
@@ -668,7 +688,7 @@ void load_url(unsigned char *url, unsigned char *prev_url, struct status *stat,
 		return;
 	}
 	skip_cache:
-	if (!casecmp(url, "proxy://", 8)) {
+	if (!casecmp(url, cast_uchar "proxy://", 8)) {
 		if (stat) {
 			stat->state = S_BAD_URL;
 			if (stat->end) stat->end(stat, stat->data);
@@ -676,7 +696,7 @@ void load_url(unsigned char *url, unsigned char *prev_url, struct status *stat,
 		return;
 	}
 	u = get_proxy(url);
-	foreach(c, queue) if (!c->detached && !strcmp(c->url, u)) {
+	foreach(c, queue) if (!c->detached && !strcmp(cast_const_char c->url, cast_const_char u)) {
 		if (c->from < position) continue;
 		if (no_compress && !c->no_compress) {
 			unsigned char *enc;
@@ -684,7 +704,7 @@ void load_url(unsigned char *url, unsigned char *prev_url, struct status *stat,
 				must_detach = 1;
 				break;
 			}
-			enc = parse_http_header(c->cache->head, "Content-Encoding", NULL);
+			enc = parse_http_header(c->cache->head, cast_uchar "Content-Encoding", NULL);
 			if (enc) {
 				mem_free(enc);
 				must_detach = 1;
@@ -735,12 +755,13 @@ void load_url(unsigned char *url, unsigned char *prev_url, struct status *stat,
 	c->no_cache = no_cache;
 	c->sock1 = c->sock2 = -1;
 	c->dnsquery = NULL;
+	c->tries = 0;
+	c->netcfg_stamp = netcfg_stamp;
+	init_list(c->statuss);
 	c->info = NULL;
 	c->buffer = NULL;
 	c->newconn = NULL;
 	c->cache = NULL;
-	c->tries = 0;
-	init_list(c->statuss);
 	c->est_length = -1;
 	c->unrestartable = 0;
 #ifdef HAVE_ANY_COMPRESSION
@@ -847,7 +868,7 @@ void detach_connection(struct status *stat, off_t pos)
 	for (i = 0; i < PRI_CANCEL; i++) l += c->pri[i];
 	if (!l) internal("detaching free connection");
 	if (l != 1 || c->cache->refcount) return;
-	shrink_memory(SH_CHECK_QUOTA);
+	shrink_memory(SH_CHECK_QUOTA, 0);
 	detach_cache_entry(c->cache);
 	c->detached = 1;
 	detach_done:
@@ -881,13 +902,6 @@ void set_timeout(struct connection *c)
 	c->timer = install_timer((c->unrestartable ? unrestartable_receive_timeout : receive_timeout) * 500, (void (*)(void *))connection_timeout_1, c);
 }
 
-#if 0
-static void reset_timeout(struct connection *c)
-{
-	if (c->timer != -1) kill_timer(c->timer), c->timer = -1;
-}
-#endif
-
 void abort_all_connections(void)
 {
 	while(queue.next != &queue) {
@@ -932,20 +946,20 @@ static struct list_head blacklist = { &blacklist, &blacklist };
 void add_blacklist_entry(unsigned char *host, int flags)
 {
 	struct blacklist_entry *b;
-	foreach(b, blacklist) if (!strcasecmp(host, b->host)) {
+	foreach(b, blacklist) if (!strcasecmp(cast_const_char host, cast_const_char b->host)) {
 		b->flags |= flags;
 		return;
 	}
-	b = mem_alloc(sizeof(struct blacklist_entry) + strlen(host) + 1);
+	b = mem_alloc(sizeof(struct blacklist_entry) + strlen(cast_const_char host) + 1);
 	b->flags = flags;
-	strcpy(b->host, host);
+	strcpy(cast_char b->host, cast_const_char host);
 	add_to_list(blacklist, b);
 }
 
 void del_blacklist_entry(unsigned char *host, int flags)
 {
 	struct blacklist_entry *b;
-	foreach(b, blacklist) if (!strcasecmp(host, b->host)) {
+	foreach(b, blacklist) if (!strcasecmp(cast_const_char host, cast_const_char b->host)) {
 		b->flags &= ~flags;
 		if (!b->flags) {
 			del_from_list(b);
@@ -958,7 +972,7 @@ void del_blacklist_entry(unsigned char *host, int flags)
 int get_blacklist_flags(unsigned char *host)
 {
 	struct blacklist_entry *b;
-	foreach(b, blacklist) if (!strcasecmp(host, b->host)) return b->flags;
+	foreach(b, blacklist) if (!strcasecmp(cast_const_char host, cast_const_char b->host)) return b->flags;
 	return 0;
 }
 
@@ -971,6 +985,7 @@ struct s_msg_dsc msg_dsc[] = {
 	{S_WAIT,		TEXT_(T_WAITING_IN_QUEUE)},
 	{S_DNS,			TEXT_(T_LOOKING_UP_HOST)},
 	{S_CONN,		TEXT_(T_MAKING_CONNECTION)},
+	{S_CONN_ANOTHER,	TEXT_(T_MAKING_CONNECTION_TO_ANOTHER_ADDRESS)},
 	{S_SOCKS_NEG,		TEXT_(T_SOCKS_NEGOTIATION)},
 	{S_SSL_NEG,		TEXT_(T_SSL_NEGOTIATION)},
 	{S_SENT,		TEXT_(T_REQUEST_SENT)},
@@ -989,6 +1004,7 @@ struct s_msg_dsc msg_dsc[] = {
 	{S_CANT_READ,		TEXT_(T_ERROR_READING_FROM_SOCKET)},
 	{S_MODIFIED,		TEXT_(T_DATA_MODIFIED)},
 	{S_BAD_URL,		TEXT_(T_BAD_URL_SYNTAX)},
+	{S_BAD_PROXY,		TEXT_(T_BAD_PROXY_SYNTAX)},
 	{S_TIMEOUT,		TEXT_(T_RECEIVE_TIMEOUT)},
 	{S_RESTART,		TEXT_(T_REQUEST_MUST_BE_RESTARTED)},
 	{S_STATE,		TEXT_(T_CANT_GET_SOCKET_STATE)},
@@ -998,6 +1014,7 @@ struct s_msg_dsc msg_dsc[] = {
 	{S_HTTP_ERROR,		TEXT_(T_BAD_HTTP_RESPONSE)},
 	{S_HTTP_100,		TEXT_(T_HTTP_100)},
 	{S_HTTP_204,		TEXT_(T_NO_CONTENT)},
+	{S_HTTPS_FWD_ERROR,	TEXT_(T_HTTPS_FWD_ERROR)},
 
 	{S_FILE_TYPE,		TEXT_(T_UNKNOWN_FILE_TYPE)},
 	{S_FILE_ERROR,		TEXT_(T_ERROR_OPENING_FILE)},
